@@ -75,14 +75,14 @@
 
 #define FSCK_RUNTIME_DIRNAME	"/run/fsck"
 
-static const char *ignored_types[] = {
+static const char *const ignored_types[] = {
 	"ignore",
 	"iso9660",
 	"sw",
 	NULL
 };
 
-static const char *really_wanted[] = {
+static const char *const really_wanted[] = {
 	"minix",
 	"ext2",
 	"ext3",
@@ -99,9 +99,9 @@ struct fsck_fs_data
 {
 	const char	*device;
 	dev_t		disk;
-	unsigned int	stacked:1,
-			done:1,
-			eval_device:1;
+	bool		stacked,
+			done,
+			eval_device;
 };
 
 /*
@@ -594,7 +594,7 @@ static int progress_active(void)
  */
 static void print_stats(struct fsck_instance *inst)
 {
-	struct timeval delta;
+	struct timeval delta = { 0 };
 
 	if (!inst || !report_stats || noexecute)
 		return;
@@ -902,7 +902,7 @@ static int wait_many(int flags)
  * If the type isn't specified by the user, then use either the type
  * specified in /etc/fstab, or DEFAULT_FSTYPE.
  */
-static int fsck_device(struct libmnt_fs *fs, int interactive)
+static int fsck_device(struct libmnt_fs *fs, int interactive, int warn_notfound)
 {
 	char *progname, *progpath;
 	const char *type;
@@ -929,6 +929,9 @@ static int fsck_device(struct libmnt_fs *fs, int interactive)
 			retval = ENOENT;
 			goto err;
 		}
+		if (warn_notfound)
+			warnx(_("fsck.%s not found; ignore %s"), type,
+					fs_get_device(fs));
 		return 0;
 	}
 
@@ -1094,7 +1097,7 @@ static int device_exists(const char *device)
 
 static int fs_ignored_type(struct libmnt_fs *fs)
 {
-	const char **ip, *type;
+	const char *const*ip, *type;
 
 	if (!mnt_fs_is_regularfs(fs))
 		return 1;
@@ -1287,7 +1290,7 @@ static int check_all(void)
 			if (!skip_root &&
 			    !fs_is_done(fs) &&
 			    !(ignore_mounted && is_mounted(fs))) {
-				status |= fsck_device(fs, 1);
+				status |= fsck_device(fs, 1, 0);
 				status |= wait_many(FLAG_WAIT_ALL);
 				if (status > FSCK_EX_NONDESTRUCT) {
 					mnt_free_iter(itr);
@@ -1350,7 +1353,7 @@ static int check_all(void)
 			/*
 			 * Spawn off the fsck process
 			 */
-			status |= fsck_device(fs, serialize);
+			status |= fsck_device(fs, serialize, 0);
 			fs_set_done(fs);
 
 			/*
@@ -1416,11 +1419,11 @@ static void __attribute__((__noreturn__)) usage(void)
 	fputs(_(" -V         explain what is being done\n"), out);
 
 	fputs(USAGE_SEPARATOR, out);
-	printf( " -?, --help     %s\n", USAGE_OPTSTR_HELP);
-	printf( "     --version  %s\n", USAGE_OPTSTR_VERSION);
+	fprintf(out, " -?, --help     %s\n", USAGE_OPTSTR_HELP);
+	fprintf(out, "     --version  %s\n", USAGE_OPTSTR_VERSION);
 	fputs(USAGE_SEPARATOR, out);
 	fputs(_("See the specific fsck.* commands for available fs-options."), out);
-	printf(USAGE_MAN_TAIL("fsck(8)"));
+	fprintf(out, USAGE_MAN_TAIL("fsck(8)"));
 	exit(FSCK_EX_OK);
 }
 
@@ -1688,7 +1691,7 @@ int main(int argc, char *argv[])
 			continue;
 		if (ignore_mounted && is_mounted(fs))
 			continue;
-		status |= fsck_device(fs, interactive);
+		status |= fsck_device(fs, interactive, interactive);
 		if (serialize ||
 		    (max_running && (num_running >= max_running))) {
 			struct fsck_instance *inst;
