@@ -1,3 +1,6 @@
+/*
+ * SPDX-License-Identifier: LGPL-2.1-or-later
+ */
 #ifndef UTIL_LINUX_STRV
 #define UTIL_LINUX_STRV
 
@@ -27,7 +30,6 @@ int strv_consume_prepend(char ***l, char *value);
 char **strv_remove(char **l, const char *s);
 
 char **strv_new(const char *x, ...);
-char **strv_new_ap(const char *x, va_list ap);
 
 static inline const char* STRV_IFNOTNULL(const char *x) {
         return x ? x : (const char *) -1;
